@@ -360,6 +360,12 @@ class People(object):
 
         expand_fields = ','.join([json_format.arg_format(f) for f in expand_fields])
 
+        # Removing the fields that are not returned in MyVoters
+        NOT_IN_MYVOTERS = ['codes', 'contribution_history', 'organization_roles']
+
+        if self.connection.db_code == 0:
+            expand_fields = [v for v in expand_fields if v not in NOT_IN_MYVOTERS]
+
         logger.info(f'Getting person with {id_type} of {id} at url {url}')
         return self.connection.get_request(url, params={'$expand': expand_fields})
 
