@@ -1,3 +1,5 @@
+import csv
+
 """
 This utility method is a generalizable method for moving files to an
 online file storage class. It is used by methods that require access
@@ -6,7 +8,7 @@ in the future, might include SFTP, and Google Cloud Storage.
 """
 
 
-def post_file(tbl, type, file_path=None, **file_storage_args):
+def post_file(tbl, type, file_path=None, quoting=csv.QUOTE_MINIMAL, **file_storage_args):
     """
     This utility method is a generalizable method for moving files to an
     online file storage class. It is used by methods that require access
@@ -22,6 +24,8 @@ def post_file(tbl, type, file_path=None, **file_storage_args):
         file_path: str
             The file path to store the file. Not required if provided with
             the **file_storage_args.
+        quoting: attr
+            The type of quoting to use for the csv.
         **kwargs: kwargs
                 Optional arguments specific to the file storage.
     `Returns:`
@@ -34,7 +38,7 @@ def post_file(tbl, type, file_path=None, **file_storage_args):
         if 'key' in file_storage_args:
             file_storage_args['key'] = file_path
 
-        return tbl.to_s3_csv(public_url=True, key=file_path, **file_storage_args)
+        return tbl.to_s3_csv(public_url=True, key=file_path, quoting=quoting, **file_storage_args)
 
     # To Do: Add in SFTP, GoogleCloud Storage
 
