@@ -5,10 +5,8 @@ from test.utils import assert_matching_tables
 import unittest
 import os
 import re
-import warnings
-import datetime
 from test.utils import validate_list
-
+from testfixtures import LogCapture
 
 # The name of the schema and will be temporarily created for the tests
 TEMP_SCHEMA = 'parsons_test2'
@@ -27,7 +25,19 @@ class TestRedshift(unittest.TestCase):
                           [2, 'John'],
                           [3, 'Sarah']])
 
+        self.tbl2 = Table([
+            ["c1", "c2", "c3", "c4", "c5", "c6", "c7"],
+            ["a", "", 1, "NA", 1.4, 1, 2],
+            ["b", "", 2, "NA", 1.4, 1, 2],
+            ["c", "", 3.4, "NA", "", "", "a"],
+            ["d", "", 5, "NA", 1.4, 1, 2],
+            ["e", "", 6, "NA", 1.4, 1, 2],
+            ["f", "", 7.8, "NA", 1.4, 1, 2],
+            ["g", "", 9, "NA", 1.4, 1, 2],
+        ])
+
         self.mapping = self.rs.generate_data_types(self.tbl)
+        self.mapping2 = self.rs.generate_data_types(self.tbl2)
 
     def test_split_full_table_name(self):
         schema, table = Redshift.split_full_table_name('some_schema.some_table')
@@ -48,8 +58,8 @@ class TestRedshift(unittest.TestCase):
 
     def test_data_type(self):
 
-        # Test smallint
-        self.assertEqual(self.rs.data_type(1, ''), 'smallint')
+        # Test int
+        self.assertEqual(self.rs.data_type(1, ''), 'int')
         # Test int
         self.assertEqual(self.rs.data_type(32769, ''), 'int')
         # Test bigint
@@ -70,7 +80,11 @@ class TestRedshift(unittest.TestCase):
         # Test correct header labels
         self.assertEqual(self.mapping['headers'], ['ID', 'Name'])
         # Test correct data types
-        self.assertEqual(self.mapping['type_list'], ['smallint', 'varchar'])
+        self.assertEqual(self.mapping['type_list'], ['int', 'varchar'])
+
+        self.assertEqual(
+            self.mapping2['type_list'],
+            ['varchar', 'varchar', 'decimal', 'varchar', "decimal", "int", "varchar"])
         # Test correct lengths
         self.assertEqual(self.mapping['longest'], [1, 5])
 
@@ -98,20 +112,23 @@ class TestRedshift(unittest.TestCase):
 
         # Test the the statement is expected
         sql = self.rs.create_sql('tmc.test', self.mapping, distkey='ID')
-        exp_sql = "create table tmc.test (\n  id smallint,\n  name varchar(5)) \ndistkey(ID) ;"
+        exp_sql = "create table tmc.test (\n  id int,\n  name varchar(5)) \ndistkey(ID) ;"
         self.assertEqual(sql, exp_sql)
 
     def test_column_validate(self):
 
-        bad_cols = ['a','a','', 'SELECT', 'asdfjkasjdfklasjdfklajskdfljaskldfjaklsdfjlaksdfjklasjdfklasjdkfljaskldfljkasjdkfasjlkdfjklasdfjklakjsfasjkdfljaslkdfjklasdfjklasjkldfakljsdfjalsdkfjklasjdfklasjdfklasdkljf'] # noqa: E501
-        fixed_cols = ['a','a_1','col_2', 'col_3', 'asdfjkasjdfklasjdfklajskdfljaskldfjaklsdfjlaksdfjklasjdfklasjdkfljaskldfljkasjdkfasjlkdfjklasdfjklakjsfasjkdfljaslkdfjkl'] # noqa: E501
+        bad_cols = ['a', 'a', '', 'SELECT', 'asdfjkasjdfklasjdfklajskdfljaskldfjaklsdfjlaksdfjklasj'
+                    'dfklasjdkfljaskldfljkasjdkfasjlkdfjklasdfjklakjsfasjkdfljaslkdfjklasdfjklasjkl'
+                    'dfakljsdfjalsdkfjklasjdfklasjdfklasdkljf']
+        fixed_cols = ['a', 'a_1', 'col_2', 'col_3', 'asdfjkasjdfklasjdfklajskdfljaskldfjaklsdfjlaks'
+                      'dfjklasjdfklasjdkfljaskldfljkasjdkfasjlkdfjklasdfjklakjsfasjkdfljaslkdfjkl']
         self.assertEqual(self.rs.column_name_validate(bad_cols), fixed_cols)
 
     def test_create_statement(self):
 
         # Assert that copy statement is expected
         sql = self.rs.create_statement(self.tbl, 'tmc.test', distkey='ID')
-        exp_sql = """create table tmc.test (\n  "id" smallint,\n  "name" varchar(5)) \ndistkey(ID) ;"""  # noqa: E501
+        exp_sql = """create table tmc.test (\n  "id" int,\n  "name" varchar(5)) \ndistkey(ID) ;"""  # noqa: E501
         self.assertEqual(sql, exp_sql)
 
         # Assert that an error is raised by an empty table
@@ -138,17 +155,12 @@ class TestRedshift(unittest.TestCase):
         os.environ['AWS_ACCESS_KEY_ID'] = prior_aws_access_key_id
         os.environ['AWS_SECRET_ACCESS_KEY'] = prior_aws_secret_access_key
 
-    def scrub_copy_tokens(self, s):
-
-        s = re.sub('=.+;', '=*HIDDEN*;', s)
-        s = re.sub('aws_secret_access_key=.+\'',
-                   'aws_secret_access_key=*HIDDEN*\'', s)
-        return s
-
     def test_copy_statement(self):
 
         sql = self.rs.copy_statement('test_schema.test', 'buck', 'file.csv',
-            aws_access_key_id='abc123', aws_secret_access_key='abc123')
+                                     aws_access_key_id='abc123',
+                                     aws_secret_access_key='abc123',
+                                     bucket_region='us-east-2')
 
         # Scrub the keys
         sql = re.sub(r'id=.+;', '*id=HIDDEN*;', re.sub(r"key=.+'", "key=*HIDDEN*'", sql))
@@ -157,16 +169,18 @@ class TestRedshift(unittest.TestCase):
                             "dateformat 'auto'", "timeformat 'auto'", "csv delimiter ','",
                             "copy test_schema.test \nfrom 's3://buck/file.csv'",
                             "'aws_access_key_*id=HIDDEN*;aws_secret_access_key=*HIDDEN*'",
-                            'emptyasnull', 'blanksasnull', 'acceptinvchars']
+                            "region 'us-east-2'", 'emptyasnull', 'blanksasnull',
+                            'acceptinvchars']
 
         # Check that all of the expected options are there:
-        [self.assertNotEqual(sql.find(o), -1) for o in expected_options]
+        [self.assertNotEqual(sql.find(o), -1, o) for o in expected_options]
 
     def test_copy_statement_columns(self):
 
         cols = ['a', 'b', 'c']
 
-        sql = self.rs.copy_statement('test_schema.test', 'buck', 'file.csv',
+        sql = self.rs.copy_statement(
+            'test_schema.test', 'buck', 'file.csv',
             aws_access_key_id='abc123', aws_secret_access_key='abc123', specifycols=cols)
 
         # Scrub the keys
@@ -205,9 +219,11 @@ class TestRedshiftDB(unittest.TestCase):
                     """
 
         other_sql = f"""
-                    create table {self.temp_schema}.test (id smallint,name varchar(5));
-                    create view {self.temp_schema}.test_view as (select * from {self.temp_schema}.test);
-                    """ # noqa: E501
+                    create table {self.temp_schema}.test (id int,name varchar(5));
+                    create view {self.temp_schema}.test_view as (
+                        select * from {self.temp_schema}.test
+                    );
+        """
 
         self.rs.query(setup_sql)
 
@@ -306,6 +322,14 @@ class TestRedshiftDB(unittest.TestCase):
         # Copy to the same table, to verify that the "drop" flag works.
         self.rs.copy(self.tbl, f'{self.temp_schema}.test_copy', if_exists='drop')
 
+        # Verify that a warning message prints when a DIST/SORT key is omitted
+        with LogCapture() as lc:
+            self.rs.copy(
+                self.tbl, f'{self.temp_schema}.test_copy', if_exists='drop', sortkey='Name')
+            desired_log = [log for log in lc.records if "optimize your queries" in log.msg][0]
+            self.assertTrue("DIST" in desired_log.msg)
+            self.assertFalse("SORT" in desired_log.msg)
+
     def test_upsert(self):
 
         # Create a target table when no target table exists
@@ -322,7 +346,8 @@ class TestRedshiftDB(unittest.TestCase):
 
         # Try to run it with a bad primary key
         self.rs.query(f"INSERT INTO {self.temp_schema}.test_copy VALUES (1, 'Jim')")
-        self.assertRaises(ValueError, self.rs.upsert, upsert_tbl, f'{self.temp_schema}.test_copy', 'ID')
+        self.assertRaises(
+            ValueError, self.rs.upsert, upsert_tbl, f'{self.temp_schema}.test_copy', 'ID')
 
         # Now try and upsert using two primary keys
         upsert_tbl = Table([['id', 'name'], [1, 'Jane']])
@@ -414,8 +439,7 @@ class TestRedshiftDB(unittest.TestCase):
         self.assertEqual(len(manifest['entries']), 3)
 
         # Validate that manifest saved to bucket
-        keys = self.s3.list_keys(self.temp_s3_bucket,
-            prefix=f'{self.temp_s3_prefix}test_manifest')
+        keys = self.s3.list_keys(self.temp_s3_bucket, prefix=f'{self.temp_s3_prefix}test_manifest')
         self.assertTrue(manifest_key in keys)
 
     def test_move_table(self):
@@ -425,8 +449,8 @@ class TestRedshiftDB(unittest.TestCase):
         self.assertTrue(self.rs.table_exists(f'{self.temp_schema}.test2'))
 
         # Run the method again, but drop original
-        self.rs.move_table(f'{self.temp_schema}.test2', f'{self.temp_schema}.test3',
-                                drop_source_table=True)
+        self.rs.move_table(
+            f'{self.temp_schema}.test2', f'{self.temp_schema}.test3', drop_source_table=True)
         self.assertFalse(self.rs.table_exists(f'{self.temp_schema}.test2'))
 
     def test_get_tables(self):
@@ -441,10 +465,11 @@ class TestRedshiftDB(unittest.TestCase):
 
         tbls_list = self.rs.get_table_stats(schema=self.temp_schema)
 
-        exp = ['database', 'schema', 'table_id', 'table', 'encoded', 'diststyle', 'sortkey1',
-               'max_varchar', 'sortkey1_enc', 'sortkey_num', 'size', 'pct_used', 'empty',
-               'unsorted', 'stats_off', 'tbl_rows', 'skew_sortkey1', 'skew_rows', 'estimated_visible_rows',
-               'risk_event', 'vacuum_sort_benefit']
+        exp = [
+            'database', 'schema', 'table_id', 'table', 'encoded', 'diststyle', 'sortkey1',
+            'max_varchar', 'sortkey1_enc', 'sortkey_num', 'size', 'pct_used', 'empty',
+            'unsorted', 'stats_off', 'tbl_rows', 'skew_sortkey1', 'skew_rows',
+            'estimated_visible_rows', 'risk_event', 'vacuum_sort_benefit']
 
         # Having some issues testing that the filter is working correctly, as it
         # takes a little bit of time for a table to show in this table and is beating
@@ -462,7 +487,6 @@ class TestRedshiftDB(unittest.TestCase):
                         f'SELECT test.id, test.name FROM {self.temp_schema}.test;')
         self.assertEqual(views.data[0], expected_row)
 
-
     def test_get_queries(self):
 
         # Validate that columns match expected columns
@@ -531,8 +555,8 @@ class TestRedshiftDB(unittest.TestCase):
         self.assertEqual(rows[0]['count'], 3)
 
         # Try with if_exists='fail'
-        self.assertRaises(ValueError,
-            self.rs.populate_table_from_query, query, dest_table, if_exists='fail')
+        self.assertRaises(
+            ValueError, self.rs.populate_table_from_query, query, dest_table, if_exists='fail')
 
     def test_duplicate_table(self):
         # Populate the source table
@@ -582,11 +606,11 @@ class TestRedshiftDB(unittest.TestCase):
     def test_get_columns(self):
         cols = self.rs.get_columns(self.temp_schema, 'test')
 
-        # id smallint,name varchar(5)
+        # id int,name varchar(5)
         expected_cols = {
             'id':   {
-                'data_type': 'smallint', 'max_length': None,
-                'max_precision': 16, 'max_scale': 0, 'is_nullable': True},
+                'data_type': 'int', 'max_length': None,
+                'max_precision': 32, 'max_scale': 0, 'is_nullable': True},
             'name': {
                 'data_type': 'character varying', 'max_length': 5,
                 'max_precision': None, 'max_scale': None, 'is_nullable': True},
@@ -722,5 +746,6 @@ class TestRedshiftDB(unittest.TestCase):
         self.rs.alter_varchar_column_widths(append_tbl, f'{self.temp_schema}.test')
         self.assertEqual(self.rs.get_columns(self.temp_schema, 'test')['name']['max_length'], 6)
 
+
 if __name__ == "__main__":
     unittest.main()
